@@ -3,8 +3,8 @@ from root_numpy import (
     tree2array, tree2rec,
     array2tree, array2root,
     list_trees, list_branches, list_structures,
-    lt, lst, lb, lst,
-    fill_array)
+    fill_array, fill_hist,
+    random_sample)
 from root_numpy_warnings import RootNumpyWarning, RootNumpyUnconvertibleWarning
 from utils import stretch, blockwise_inner_join
 from info import __version__
@@ -17,10 +17,11 @@ __all__ = [
     'array2tree',
     'array2root',
     'fill_array',
+    'fill_hist',
+    'random_sample',
     'list_trees',
     'list_branches',
     'list_structures',
-    'lt', 'lb', 'lst',
     'stretch',
     'blockwise_inner_join',
     'RootNumpyWarning',
